@@ -17,9 +17,10 @@ SCCI functionalities shared between different iRMC modules.
 """
 
 import functools
+import re
 import time
-import xml.etree.ElementTree as ET
 
+import defusedxml.ElementTree as ET
 import requests
 import six
 
@@ -28,6 +29,14 @@ from scciclient.irmc import snmp
 
 DEBUG = False
 
+S4_PATTERN = re.compile(r"^iRMC\s*S4$")
+S5_PATTERN = re.compile(r"^iRMC\s*S5$")
+FW_VERSION_HEAD_PATTERN = re.compile(r"^\d[\d.]*")
+FW_VERSION_TAIL_PATTERN = re.compile(r"[a-zA-Z]*$")
+
+S4_FD_SUPPORT_UPPER = 9.21
+S5_FD_SUPPORT_UPPER = 1.25
+
 
 class SCCIError(Exception):
     """SCCI Error
@@ -74,7 +83,7 @@ class SCCISessionTimeout(SCCIError):
 
 
 """
-List of iRMC S4 supported SCCI commands
+List of iRMC S4/S5 supported SCCI commands
 
 SCCI
 OpCode  SCCI Command String      Description
@@ -90,6 +99,14 @@ OpCode  SCCI Command String      Description
 0x0203  ResetFirmware  Perform a BMC Reset
 0x0251  ConnectRemoteFdImage     Connect or Disconnect a Floppy Disk image on a
                                  Remote Image Mount (NFS or CIFS Share )
+
+                                 This command was deprecated at following
+                                 version:
+                                   iRMC S4: 9.62F
+                                            (9.21F still supports virtual FD)
+                                   iRMC S5: 1.60P
+                                            (1.25P still supports virtual FD)
+
 0x0252  ConnectRemoteCdImage     Connect or Disconnect a CD/DVD .iso image on a
                                  Remote Image Mount (NFS or CIFS Share )
 0x0253  ConnectRemoteHdImage     Connect or Disconnect a Hard Disk image on a
@@ -247,7 +264,7 @@ def get_share_type(share_type):
 
 
 def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
-             client_timeout=60, do_async=True, **kwargs):
+             client_timeout=60, do_async=True, verify=True, **kwargs):
     """execute SCCI command
 
     This function calls SCCI server modules
@@ -259,6 +276,10 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
     :param auth_method: irmc_username
     :param client_timeout: timeout for SCCI operations
     :param do_async: async call if True, sync call otherwise
+    :param verify: (optional) Either a boolean, in which case it
+                    controls whether we verify the server's TLS certificate,
+                    or a string, in which case it must be a path to
+                    a CA bundle to use. Defaults to ``True``.
     :returns: requests.Response from SCCI server
     :raises: SCCIInvalidInputError if port and/or auth_method params
              are invalid
@@ -289,12 +310,26 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
                 data = file.read()
             config_type = '/biosupdate'
         else:
+            # For EJECT command, validate parameters to handle abnormal case
+            if check_eject_cd_cmd(cmd):
+                if not validate_params_cd_fd("cmd_cd", protocol,
+                                             host, auth_obj,
+                                             do_async, client_timeout,
+                                             verify):
+                    return
+            if check_eject_fd_cmd(cmd):
+                if not validate_params_cd_fd("cmd_fd", protocol,
+                                             host, auth_obj,
+                                             do_async, client_timeout,
+                                             verify):
+                    return
             data = cmd
             config_type = '/config'
+
         r = requests.post(protocol + '://' + host + config_type,
                           data=data,
                           headers=header,
-                          verify=False,
+                          verify=verify,
                           timeout=client_timeout,
                           allow_redirects=False,
                           auth=auth_obj)
@@ -343,8 +378,125 @@ def scci_cmd(host, userid, password, cmd, port=443, auth_method='basic',
         raise SCCIClientError(requests_exception)
 
 
+def validate_params_cd_fd(cmd_type, protocol, host, auth_obj,
+                          do_async, client_timeout, verify):
+    """Validate parameters of CD/DVD or FD Image Virtual Media in iRMC
+
+    If one of parameters (ImageServer, ImageShareName or ImageName) set in
+    ServerView Config Space is empty, and you try to eject virtual FD/CD,
+    iRMC returns error. This function determines whether iRMC doesn't return
+    error when you try to eject virtual FD/CD.
+    :param cmd_type: command type has value switch between "cmd_cd" or "cmd_fd"
+    :param protocol:
+    :param host: hostname or IP of iRMC
+    :param auth_obj: irmc userid/password
+    :param do_async: async call if True, sync call otherwise
+    :param client_timeout: timeout for SCCI operations
+    :param verify: Either a boolean, in which case it
+                   controls whether we verify the server's TLS certificate,
+                   or a string, in which case it must be a path to
+                   a CA bundle to use.
+    :return: False if one of param is null. Otherwise, returns True.
+    """
+
+    if cmd_type == "cmd_cd":
+        oe_image_server = "1A60"
+        oe_image_server_share_name = "1A65"
+        oe_image_name = "1A66"
+    else:
+        oe_image_server = "1A50"
+        oe_image_server_share_name = "1A55"
+        oe_image_name = "1A56"
+
+    try:
+        param = {'P45': '1', 'SAVE_DATA': '1'}
+        header = {'Content-type': 'application/x-www-form-urlencoded'}
+        r = requests.get(protocol + '://' + host + '/iRMC_Settings.pre',
+                         params=param,
+                         headers=header,
+                         verify=verify,
+                         timeout=client_timeout,
+                         allow_redirects=False,
+                         auth=auth_obj)
+        if not do_async:
+            time.sleep(5)
+
+        if DEBUG:
+            print("---------------------------")
+            print("Current iRMC configuration:")
+            print(r.url)
+
+        if r.status_code not in (200, 201):
+            raise SCCIClientError(
+                ('HTTP PROTOCOL ERROR, STATUS CODE = %s' %
+                 str(r.status_code)))
+
+        result = r.text
+        cmdseq = ET.fromstring(result)
+        cfg_dict = {}
+        for cmd_tag in cmdseq.iter(tag='CMD'):
+            oe = cmd_tag.get('OE')
+            data = cmd_tag.find('DATA').text
+            cfg_dict[oe] = data
+
+        if DEBUG:
+            print("Server: ", cfg_dict[oe_image_server])
+            print("Share Name: ", cfg_dict[oe_image_server_share_name])
+            print("Image Name: ", cfg_dict[oe_image_name])
+            print("---------------------------")
+
+        if (cfg_dict[oe_image_server] is None) or \
+                (cfg_dict[oe_image_server_share_name] is None) or \
+                (cfg_dict[oe_image_name] is None):
+            return False
+
+    except ET.ParseError as parse_error:
+        raise SCCIClientError(parse_error)
+    except requests.exceptions.RequestException as requests_exception:
+        raise SCCIClientError(requests_exception)
+
+    return True
+
+
+def check_eject_cd_cmd(xml_cmd):
+    """To check command is MOUNT or UNMOUNT
+
+    :param xml_cmd: the command
+    :return: true if this is UNMOUNT command. Otherwise, return false.
+    """
+
+    try:
+        cmdseq = ET.fromstring(xml_cmd.strip())
+        cmd = cmdseq.find("./CMD")
+        data = cmd.find("./DATA")
+        if cmd.get("OC") == "ConnectRemoteCdImage" and \
+                cmd.get("Type") == "SET" and data.text == "0":
+            return True
+    except ET.ParseError as parse_error:
+        raise SCCIClientError(parse_error)
+    return False
+
+
+def check_eject_fd_cmd(xml_cmd):
+    """To check command is MOUNT or UNMOUNT
+
+    :param xml_cmd: the command
+    :return: true if this is UNMOUNT command. Otherwise, return false.
+    """
+    try:
+        cmdseq = ET.fromstring(xml_cmd.strip())
+        cmd = cmdseq.find("./CMD")
+        data = cmd.find("./DATA")
+        if cmd.get("OC") == "ConnectRemoteFdImage" and \
+                cmd.get("Type") == "SET" and data.text == "0":
+            return True
+    except ET.ParseError as parse_error:
+        raise SCCIClientError(parse_error)
+    return False
+
+
 def get_client(host, userid, password, port=443, auth_method='basic',
-               client_timeout=60, **kwargs):
+               client_timeout=60, verify=True, **kwargs):
     """get SCCI command partial function
 
     This function returns SCCI command partial function
@@ -354,12 +506,17 @@ def get_client(host, userid, password, port=443, auth_method='basic',
     :param port: port number of iRMC
     :param auth_method: irmc_username
     :param client_timeout: timeout for SCCI operations
+    :param verify: (optional) Either a boolean, in which case it
+                    controls whether we verify the server's TLS certificate,
+                    or a string, in which case it must be a path to
+                    a CA bundle to use. Defaults to ``True``.
     :returns: scci_cmd partial function which takes a SCCI command param
     """
 
     return functools.partial(scci_cmd, host, userid, password,
                              port=port, auth_method=auth_method,
-                             client_timeout=client_timeout, **kwargs)
+                             client_timeout=client_timeout,
+                             verify=verify, **kwargs)
 
 
 def get_virtual_cd_set_params_cmd(remote_image_server,
@@ -426,7 +583,7 @@ def get_virtual_fd_set_params_cmd(remote_image_server,
 
 
 def get_report(host, userid, password,
-               port=443, auth_method='basic', client_timeout=60):
+               port=443, auth_method='basic', client_timeout=60, verify=True):
     """get iRMC report
 
     This function returns iRMC report in XML format
@@ -436,6 +593,10 @@ def get_report(host, userid, password,
     :param port: port number of iRMC
     :param auth_method: irmc_username
     :param client_timeout: timeout for SCCI operations
+    :param verify: (optional) Either a boolean, in which case it
+                    controls whether we verify the server's TLS certificate,
+                    or a string, in which case it must be a path to
+                    a CA bundle to use. Defaults to ``True``.
     :returns: root element of SCCI report
     :raises: ISCCIInvalidInputError if port and/or auth_method params
              are invalid
@@ -458,7 +619,7 @@ def get_report(host, userid, password,
 
     try:
         r = requests.get(protocol + '://' + host + '/report.xml',
-                         verify=False,
+                         verify=verify,
                          timeout=(10, client_timeout),
                          allow_redirects=False,
                          auth=auth_obj)
@@ -487,7 +648,6 @@ def get_sensor_data_records(report):
     """
 
     sensor = report.find("./System/SensorDataRecords")
-    # ET.dump(sensor[0])
     return sensor
 
 
@@ -500,10 +660,56 @@ def get_irmc_version(report):
     """
 
     version = report.find("./System/ManagementControllers/iRMC")
-    # ET.dump(version[0])
     return version
 
 
+def get_irmc_version_str(report):
+    """extract iRMC OS and iRMC firmware version from SCCI report
+
+    This function returns iRMC OS name and iRMC firmware version
+    :param report: SCCI report element
+    :returns: a tuple of string (iRMC OS name, iRMC FW version)
+    """
+
+    version = get_irmc_version(report)
+    return version.get('Name'), version.find('Firmware').text
+
+
+def support_virtual_fd_str(irmc_os, fw_version):
+    """determine iRMC supports virtual floppy disk
+
+    This function determines whether iRMC supports virtual floppy disk
+    based on provided iRMC OS & iRMC firmware version
+    :param irmc_os: string representing iRMC OS
+    :param fw_version: string representing iRMC firmware version
+    :returns: boolean
+    """
+
+    version_head = FW_VERSION_HEAD_PATTERN.match(fw_version)
+
+    if S4_PATTERN.match(irmc_os):
+        if version_head and float(version_head.group()) <= S4_FD_SUPPORT_UPPER:
+            return True
+    elif S5_PATTERN.match(irmc_os):
+        if version_head and float(version_head.group()) <= S5_FD_SUPPORT_UPPER:
+            return True
+
+    return False
+
+
+def support_virtual_fd(report):
+    """determine iRMC supports virtual floppy disk
+
+    This function determines whether iRMC supports virtual floppy disk
+    based on SCCI report
+    :param report: SCCI report element
+    :returns: boolean
+    """
+
+    irmc_os, fwv = get_irmc_version_str(report)
+    return support_virtual_fd_str(irmc_os, fwv)
+
+
 def get_essential_properties(report, prop_keys):
     """get essential properties
 
@@ -681,6 +887,10 @@ def get_firmware_upgrade_status(irmc_info, upgrade_type):
           'irmc_port': 80 or 443, default is 443,
           'irmc_auth_method': 'basic' or 'digest', default is 'digest',
           'irmc_client_timeout': timeout, default is 60,
+          'irmc_verify_ca': (optional) Either a boolean, in which case it
+                            controls whether we verify the server's TLS
+                            certificate, or a string, in which case it must be
+                            a path to a CA bundle to use. Defaults to ``True``.
           ...
         }
     :param upgrade_type: flag to check upgrade with bios or irmc
@@ -695,6 +905,7 @@ def get_firmware_upgrade_status(irmc_info, upgrade_type):
     port = irmc_info.get('irmc_port', 443)
     auth_method = irmc_info.get('irmc_auth_method', 'digest')
     client_timeout = irmc_info.get('irmc_client_timeout', 60)
+    verify = irmc_info.get('irmc_verify_ca', True)
 
     auth_obj = None
     try:
@@ -714,7 +925,7 @@ def get_firmware_upgrade_status(irmc_info, upgrade_type):
         elif upgrade_type == 'irmc':
             upgrade_type = '/irmcprogress'
         r = requests.get(protocol + '://' + host + upgrade_type,
-                         verify=False,
+                         verify=verify,
                          timeout=(10, client_timeout),
                          allow_redirects=False,
                          auth=auth_obj)
@@ -728,3 +939,5 @@ def get_firmware_upgrade_status(irmc_info, upgrade_type):
         return upgrade_status_xml
     except ET.ParseError as parse_error:
         raise SCCIClientError(parse_error)
+    except requests.RequestException as requests_exception:
+        raise SCCIClientError(requests_exception)
