@@ -223,7 +223,7 @@ class SCCITestCase(testtools.TestCase):
             'https://' + self.irmc_address + '/config',
             data=scci.POWER_ON,
             headers={'Content-type': 'application/x-www-form-urlencoded'},
-            verify=False,
+            verify=True,
             timeout=self.irmc_client_timeout,
             allow_redirects=False,
             auth=mock_requests.auth.HTTPBasicAuth(self.irmc_username,
@@ -1026,9 +1026,9 @@ class SCCITestCase(testtools.TestCase):
         result = scci.get_raid_fgi_status(report_fake)
         self.assertEqual(result, fgi_status_expect)
 
-    @mock.patch('scciclient.irmc.scci.requests')
-    def test_fail_get_bios_firmware_status(self, mock_requests):
-        mock_requests.get.return_value = mock.Mock(
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_fail_get_bios_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
             status_code=404,
             text="""</head>
             <body>
@@ -1049,9 +1049,9 @@ class SCCITestCase(testtools.TestCase):
                           scci.get_firmware_upgrade_status, self.irmc_info,
                           upgrade_type=upgrade_type)
 
-    @mock.patch('scciclient.irmc.scci.requests')
-    def test_success_get_bios_firmware_status(self, mock_requests):
-        mock_requests.get.return_value = mock.Mock(
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_success_get_bios_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
             return_value='ok',
             status_code=200,
             text="""<?xml version="1.0" encoding="UTF-8"?>
@@ -1069,9 +1069,9 @@ class SCCITestCase(testtools.TestCase):
         self.assertEqual(expected_severity, result.find("./Severity").text)
         self.assertEqual(expected_message, result.find("./Message").text)
 
-    @mock.patch('scciclient.irmc.scci.requests')
-    def test_fail_get_irmc_firmware_status(self, mock_requests):
-        mock_requests.get.return_value = mock.Mock(
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_fail_get_irmc_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
             status_code=404,
             text="""</head>
             <body>
@@ -1092,9 +1092,9 @@ class SCCITestCase(testtools.TestCase):
                           scci.get_firmware_upgrade_status, self.irmc_info,
                           upgrade_type=upgrade_type)
 
-    @mock.patch('scciclient.irmc.scci.requests')
-    def test_success_get_irmc_firmware_status(self, mock_requests):
-        mock_requests.get.return_value = mock.Mock(
+    @mock.patch('scciclient.irmc.scci.requests.get')
+    def test_success_get_irmc_firmware_status(self, mock_requests_get):
+        mock_requests_get.return_value = mock.Mock(
             return_value='ok',
             status_code=200,
             text="""<?xml version="1.0" encoding="UTF-8"?>
